package com.grack.hdcp;

import java.security.InvalidParameterException;

public class HdcpKeyGenerator {

	private HdcpSystem system;

	private SecretKey[] systemPrivateKey;

	public HdcpKeyGenerator(HdcpSystem system, SecretKey[] systemPrivateKey) {
		super();
		this.system = system;
		this.systemPrivateKey = systemPrivateKey;
		
		if (systemPrivateKey.length != system.getKeyArraySize())
			throw new IllegalArgumentException("There must be exactly the same number of secret keys as the key array size");

		for ( int i = 0; i < systemPrivateKey.length; i++) {
			for ( int j = 0; j < systemPrivateKey.length; j++) {
				long key1 = systemPrivateKey[i].calculateKey(new PublicKey(system, new int[] { j + 1 }, true));
				long key2 = systemPrivateKey[j].calculateKey(new PublicKey(system, new int[] { i + 1 }, true));
				
				if (key1 != key2)
					throw new IllegalArgumentException(String.format("Inconsistent state for key generator: mismatch between %d and %d", i, j));
			}
		}
	}

	public HdcpDevice createDevice(PublicKey publicKey) {
		long[] secretKey = new long[system.getKeyArraySize()];
		for ( int i = 0; i < system.getKeyArraySize(); i++ )
			secretKey[i] = systemPrivateKey[i].calculateKey(publicKey);
		
		return new HdcpDevice(system, new SecretKey(secretKey), publicKey);
	}
}
